/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton.org

Copyright (c) 2002, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/***************************************************
 * This is a minimal grep type program for my      *
 * supersmall floppy/tftp distribution             *
 * it only recognizes ^ and $ as special symbols   *
 * (the beginning and end of a line, respectively) *
 * I wrote it to parse simple config files without *
 * having to include grep because of its size.     *
 ***************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#define kMaxLineLen 1024

int process_file(char *filename);

int main(int argc, char **argv)
{
	int ret;
	
	if( argc < 2 )
		return 2;

	ret = process_file(argv[1]);

	return 0;
}

int process_file(char *pattern)
{
	int linelen, patlen, startFlag = 0, endFlag = 0;
	int err = 0;
	char *find;
	char *line = (char *)malloc(sizeof(char)*(kMaxLineLen+1));
	if(!line)
		return ENOMEM;
	
	patlen = strlen(pattern);
	if( !patlen )
		goto cleanup; // empty pattern, fuck it

	if( *pattern == '^' )
	{
		startFlag = 1;
		pattern++;
		patlen--;
	}

	if( pattern[patlen-1] == '$' )
	{
		endFlag = 1;
		patlen--;
		pattern[patlen] = 0;
	}
	
	//printf("pattern is %s,  startFlag is %d, endFlag is %d\n",
	//	pattern, startFlag, endFlag );

	while(!feof(stdin))
	{
		if( ! fgets(line,kMaxLineLen,stdin) )
		{
			// see if it was an error or eof
			err = ferror(stdin);
			if( err )
				goto cleanup;
			else
				break;
		}
		linelen = strlen(line);
		// kill the newline if there is one
		if( line[linelen-1] == '\n' )
			line[--linelen] = 0;

		// check for the pattern
		find = strstr(line,pattern);
		if( find )
		{
			if( 
				(!startFlag || find == line) &&
				(!endFlag || (find+patlen==line+linelen ))
				)
			{
				// found it, print the line to stdout
				puts( line );
			}
		}
	}
cleanup:
	// put the char we modified back if endFlag is true
	if( endFlag )
		pattern[patlen] = '$';

	if(line)
		free( (void *)line );
	return err;
}

